/*
 * #%L
 * EUGene :: EUGene
 * 
 * $Id: JavaGeneratorUtil.java 1014 2010-11-28 20:13:58Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/eugene/tags/eugene-2.3.1/eugene/src/main/java/org/nuiton/eugene/java/JavaGeneratorUtil.java $
 * %%
 * Copyright (C) 2004 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.eugene.java;

import org.nuiton.eugene.EugeneStereoTypes;
import org.nuiton.eugene.EugeneTagValues;
import org.nuiton.eugene.GeneratorUtil;
import org.nuiton.eugene.models.object.ObjectModel;
import org.nuiton.eugene.models.object.ObjectModelClassifier;
import org.nuiton.eugene.models.object.ObjectModelElement;
import org.nuiton.eugene.models.object.ObjectModelModifier;
import org.nuiton.eugene.models.object.ObjectModelOperation;
import org.nuiton.eugene.models.object.ObjectModelParameter;

/**
 * Utility class for pure java templates.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 2.0.2
 */
public class JavaGeneratorUtil extends GeneratorUtil {

    public static final String OPERATION_GETTER_DEFAULT_PREFIX = "get";

    public static final String OPERATION_GETTER_BOOLEAN_PREFIX = "is";

    /**
     * dependency to add constants in interfaces via an enumeration.
     * <p/>
     * The literals of enumeration will be the value, and constant names will
     * be generated by transformers.
     *
     * @since 2.0.2
     */
    public static final String DEPENDENCIES_CONSTANTS = "constants";

    /**
     * Duplicates in the {@code target} classifier the given {@code operations}
     * using a {@code transformer} to modify model.
     *
     * @param transformer    the transformer to use
     * @param operations     operations to duplicate
     * @param target         where to duplicate operations
     * @param useVisibility  flag to use operation visibilty to compute his
     *                       modifiers
     * @param extraModifiers scopes to apply to all operations
     */
    public static void cloneOperations(ObjectModelTransformerToJava transformer,
                                       Iterable<ObjectModelOperation> operations,
                                       ObjectModelClassifier target,
                                       boolean useVisibility,
                                       ObjectModelModifier... extraModifiers) {

        for (ObjectModelOperation op : operations) {
            ObjectModelOperation resultOp;
            ObjectModelModifier[] modifiers;
            if (useVisibility) {

                // compute visibility modifer
                String visibility = op.getVisibility();
                ObjectModelModifier modifier =
                        ObjectModelModifier.toValue(visibility);

                int length = extraModifiers.length;
                if (length == 0) {
                    modifiers = new ObjectModelModifier[]{modifier};
                } else {
                    modifiers = new ObjectModelModifier[length + 1];
                    modifiers[0] = modifier;
                    System.arraycopy(extraModifiers, 0, modifiers, 1, length);
                }
            } else {

                // just use the incoming modifiers
                modifiers = extraModifiers;
            }

            resultOp = transformer.addOperation(target,
                                                op.getName(),
                                                op.getReturnType(),
                                                modifiers
            );
            for (ObjectModelParameter param : op.getParameters()) {
                transformer.addParameter(resultOp,
                                         param.getType(),
                                         param.getName()
                );
            }
            for (String exception : op.getExceptions()) {
                transformer.addException(resultOp, exception);
            }
        }
    }

    /**
     * Check if the given classifier has the
     * {@link EugeneStereoTypes#STEREOTYPE_BEAN} stereotype.
     *
     * @param classifier classifier to test
     * @return {@code true} if stereotype was found, {@code false otherwise}
     * @see EugeneStereoTypes#STEREOTYPE_BEAN
     */
    public static boolean hasBeanStereotype(ObjectModelClassifier classifier) {
        return classifier.hasStereotype(EugeneStereoTypes.STEREOTYPE_BEAN);
    }

    /**
     * Cherche et renvoie le prefixe i18n à utiliser sur cet element, sinon sur
     * le model.
     *
     * @param element l'élément à tester
     * @param model   le modele utilisé
     * @return le prefix i18n ou <code>null</code> si non spécifié
     * @since 2.3
     */
    public static String getI18nPrefixTagValue(ObjectModelElement element,
                                               ObjectModel model) {
        return findTagValue(EugeneTagValues.TAG_I18N_PREFIX, element, model);
    }

    /**
     * Obtain the value of the {@link EugeneTagValues#TAG_NO_PCS}
     * tag value on the given model or classifier.
     * <p/>
     * It will first look on the model, and then in the given classifier.
     *
     * @param model      model to seek
     * @param classifier classifier to seek
     * @return the none empty value of the found tag value or {@code null} if not found nor empty.
     * @see EugeneTagValues#TAG_NO_PCS
     * @since 2.3
     */
    public static String getNoPCSTagValue(ObjectModel model, ObjectModelClassifier classifier) {
        String value = findTagValue(EugeneTagValues.TAG_NO_PCS, classifier, model);
        return value;
    }

    /**
     * Obtain the value of the {@link EugeneTagValues#TAG_CONSTANT_PREFIX}
     * tag value on the given model or classifier.
     * <p/>
     * It will first look on the model, and then in the given classifier.
     *
     * @param model      model to seek
     * @param classifier classifier to seek
     * @return the none empty value of the found tag value or {@code null} if not found nor empty.
     * @see EugeneTagValues#TAG_CONSTANT_PREFIX
     * @since 2.3
     */
    public static String getConstantPrefixTagValue(ObjectModel model,
                                                   ObjectModelClassifier classifier) {
        String value = findTagValue(EugeneTagValues.TAG_CONSTANT_PREFIX, classifier, model);
        return value;
    }
}
